@@ -87,6 +87,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             pub use checkout_core::error::new_server_error;
             pub use checkout_core::server::Server;
             pub use checkout_core::internal_context::InternalContext;
+            pub use checkout_core::pubsub_server;
 
             use std::env;
             use tokio::task;
@@ -139,16 +140,15 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
             }
 
             #[derive(Debug)]
-            struct Message(String);
+            pub struct IdWrapper(String);
 
-            impl cloud_pubsub::FromPubSubMessage for Message {
+            impl cloud_pubsub::FromPubSubMessage for IdWrapper {
                 fn from(message: cloud_pubsub::EncodedMessage) -> Result<Self, cloud_pubsub::error::Error> {
                     match message.decode() {
                         Ok(bytes) => {
-                            let raw_string = String::from_utf8_lossy(&bytes).into_owned();
-                            let val: serde_json::Value = serde_json::from_str(&raw_string).unwrap();
-                            Ok(Message(val.as_str().unwrap().to_string()))
-                        },
+                            let val: serde_json::Value = serde_json::from_slice(&bytes).unwrap();
+                            Ok(IdWrapper(val.as_str().unwrap().to_string()))
+                        }
                         Err(e) => Err(cloud_pubsub::error::Error::from(e)),
                     }
                 }
@@ -164,7 +164,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     while subscription.client().is_running() {
                         println!("pulling payment processing messages");
 
-                        match subscription.get_messages::<Message>().await {
+                        match subscription.get_messages::<IdWrapper>().await {
                             Ok(messages) => {
                                 for (result, ack_id) in messages {
                                     match result {
@@ -703,7 +703,10 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                 let pubsub_client = internal_context.pubsub_client.clone();
                 pubsub_client.spawn_token_renew(checkout_server::Duration::from_secs(15 * 60));
 
-                checkout_server::start_payment_processing(internal_context.clone(), context_provider.clone());
+                // start experiment
+                // checkout_server::start_payment_processing(internal_context.clone(), context_provider.clone());
+                checkout_server::pubsub_server::start_payment_processing(internal_context.clone(), &self);
+                // end experiment
 
                 checkout_server::HttpServerStarter::new(
                     &checkout_server::ConfigDropshot {
